use std::{error::Error as StdError, fmt::Display};

use axum::{
    response::{IntoResponse, Response},
    Json,
};
use serde::Serialize;

use super::status::Status;

pub struct ResOK<T>(pub Option<T>)
where
    T: Serialize;

impl<T> IntoResponse for ResOK<T>
where
    T: Serialize,
{
    fn into_response(self) -> Response {
        let ResOK(data) = self;
        let status = Status::OK(data);

        Json(status.to_reply()).into_response()
    }
}

#[derive(Debug)]
pub enum ResErr {
    Error(i32, String),
    ErrParams(Option<String>),
    ErrAuth(Option<String>),
    ErrPerm(Option<String>),
    ErrNotFound(Option<String>),
    ErrSystem(Option<String>),
    ErrData(Option<String>),
    ErrService(Option<String>),
    ErrSocial(Option<String>),
}

use ResErr::*;

impl IntoResponse for ResErr {
    fn into_response(self) -> Response {
        let status = match self {
            Error(code, msg) => Status::<()>::Err(code, msg),
            ErrParams(msg) => {
                let code = 10000;

                match msg {
                    Some(v) => Status::<()>::Err(code, v),
                    None => Status::<()>::Err(code, String::from("参数错误")),
                }
            }
            ErrAuth(msg) => {
                let code = 20000;

                match msg {
                    Some(v) => Status::<()>::Err(code, v),
                    None => Status::<()>::Err(code, String::from("未授权，请先登录")),
                }
            }
            ErrPerm(msg) => {
                let code = 30000;

                match msg {
                    Some(v) => Status::<()>::Err(code, v),
                    None => Status::<()>::Err(code, String::from("权限不足")),
                }
            }
            ErrNotFound(msg) => {
                let code = 40000;

                match msg {
                    Some(v) => Status::<()>::Err(code, v),
                    None => Status::<()>::Err(code, String::from("数据不存在")),
                }
            }
            ErrSystem(msg) => {
                let code = 50000;

                match msg {
                    Some(v) => Status::<()>::Err(code, v),
                    None => Status::<()>::Err(code, String::from("内部服务器错误，请稍后重试")),
                }
            }
            ErrData(msg) => {
                let code = 60000;

                match msg {
                    Some(v) => Status::<()>::Err(code, v),
                    None => Status::<()>::Err(code, String::from("数据异常")),
                }
            }
            ErrService(msg) => {
                let code = 70000;

                match msg {
                    Some(v) => Status::<()>::Err(code, v),
                    None => Status::<()>::Err(code, String::from("服务异常")),
                }
            }
            ErrSocial(msg) => {
                let code = 80000;
                match msg {
                    Some(v) => Status::<()>::Err(code, v),
                    None => Status::<()>::Err(code, String::from("社交服务异常")),
                }
            }
        };

        Json(status.to_reply()).into_response()
    }
}

// impl Display for ResErr {
//     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
//         // write!(f, "{:?}", self.message)
//         match self {
//             Error(value, message) => write!(f, "{} - {:?}", value, message),
//             ErrParams(message)
//             | ErrAuth(message)
//             | ErrPerm(message)
//             | ErrNotFound(message)
//             | ErrSystem(message)
//             | ErrData(message)
//             | ErrService(message)
//             | ErrSocial(message) => write!(f, "{:?}", message),
//         }
//     }
// }

// impl StdError for ResErr {
//     fn cause(&self) -> Option<&dyn StdError> {
//         match self {
//             Error(_, _) => None,
//             ErrParams(_)
//             | ErrAuth(_)
//             | ErrPerm(_)
//             | ErrNotFound(_)
//             | ErrSystem(_)
//             | ErrData(_)
//             | ErrService(_)
//             | ErrSocial(_) => None,
//         }
//     }
    
//     fn source(&self) -> Option<&(dyn StdError + 'static)> {
//         None
//     }
    
//     fn description(&self) -> &str {
//         match self {
//             Error(value, message) => &format!("Error, {} - {}", value, message),
//             ErrParams(message) => &format!("ErrParams, {}", message),
//             ErrAuth(message) => &format!("ErrAuth, {}", message),
//             ErrPerm(message) => &format!("ErrPerm, {}", message),
//             ErrNotFound(message) => &format!("ErrNotFound, {}", message),
//             ErrSystem(message) => &format!("ErrSystem, {}", message),
//             ErrData(message) => &format!("ErrData, {}", message),
//             ErrService(message) => &format!("ErrService, {}", message),
//             ErrSocial(message) => &format!("ErrSocial, {}", message),
//         }
//     }
    
//     fn provide<'a>(&'a self, request: &mut std::error::Request<'a>) {
//         match self {
//             ResErr::ErrSystem(e) => request.provide_ref(e),
//             ResErr::ErrData(e) => request.provide_ref(e),
//             ResErr::ErrService(e) => request.provide_ref(e),
//             ResErr::ErrSocial(e) => request.provide_ref(e),
//             ResErr::ErrNotFound(e) => request.provide_ref(e),
//             ResErr::ErrPerm(e) => request.provide_ref(e),
//             ResErr::ErrAuth(e) => request.provide_ref(e),
//             ResErr::ErrParams(e) => request.provide_ref(e),
//             ResErr::Error(_, e) => request.provide_ref(e),
//     }
// }

pub type ResResult<T> = std::result::Result<T, ResErr>;
